#ifndef MAXLIGHTS
#define MAXLIGHTS 5
#endif

#ifndef NUMLIGHTS
#define NUMLIGHTS 0
#endif 

#define FT_PCF 0
#define FT_3x3GAUSS 2
#define FT_16xPOISSON 3

//must be an array of NUMLIGHTS values, that define the lighttype
#ifndef LIGHTTYPES
#define LIGHTTYPES 0
#endif

//#define NOFILTERING

static const int LIGHTTYPE[] = { LIGHTTYPES };

#include "Lighting_Shared.fxh"

float LightAttenuation(const int l, const float dist)
{
	//[branch] if (att <= 0.5) //linear and constant attenuation
	//{
	//	float c = (0.5f - att) * 2.0f;
	//	float l = att * 2.0f;
	//	return 1 / (c + l * dist);
	//}
	//else
	//{
	//	float a = att - 0.5f;
	//	float l = (0.5f - a) * 2.0f;
	//	float q = a * 2.0f;
	//	return 1 / (l * dist + q * dist * dist);
	//}

	//float d = min(1, dist / g_Lights[l].Radius);
	//float att = 1 - dot(float3(0, d, d * d), g_Lights[l].Attenuation);
	//return att;

	//calculate attenuation from real distance
	float att = 1.0f / dot(float3(1, dist, dist * dist), g_Lights[l].Attenuation);
	//calculate attenuation from range
	float rd = 1 - min(1, dist / g_Lights[l].Radius);
	//interpolate to rangeatt in last 25%
	float s = saturate((1 - rd - 0.75f) * 4);
	float rangeatt = rd * rd;
	return lerp(att, rangeatt, s);
}

//returns diffuse and spcular lighting
float2 PhongLighting(const float3 Normal, const float3 LightVec, const float3 ReflVec, const float Shinyness)
{
	//get factor of lightstrength by dot(L, N)
	float DotLN = dot(Normal, LightVec);
	
	//calc reflection vector
	//float3 ReflVec = normalize(2 * DotLN * Normal - LightVec);

	//calc specular factor
	//float spec = Shinyness > 0 ? pow(saturate(dot(ReflVec, ViewVec)), Shinyness) : 0;
	float spec = Shinyness > 0 ? pow(saturate(dot(ReflVec, LightVec)), Shinyness) : 0;
	
	return float2(saturate(DotLN), spec);
}

float Do2DShadowMapLookup(const int l, float2 tc, float depth)
{
	float Shadow = 0;

#ifndef NOFILTERING
	[branch] if (g_Lights[l].SMFilter == FT_3x3GAUSS)
	{
		for (int y = -1; y <= 1; y++)
		{
			for (int x = -1; x <= 1; x++)
			{
				Shadow += Gauss3x3Weights[x + 1 + (y + 1) * 3] * g_tx2DShadowMaps[l].SampleCmpLevelZero(g_sShadowMap, tc + float2(x, y) * g_Lights[l].InvSMSize, depth);					
			}
		}
	}
	else if (g_Lights[l].SMFilter == FT_16xPOISSON)
	{
		for(int i = 0; i < 16; i++)
		{
			Shadow += g_tx2DShadowMaps[l].SampleCmpLevelZero(g_sShadowMap, tc + PoissonOffsets[i] * g_Lights[l].InvSMSize, depth);
		}

		Shadow *= 1 / 16.0f;
	}
	else
#endif
	{
		Shadow = g_tx2DShadowMaps[l].SampleCmpLevelZero(g_sShadowMap, tc, depth);
	}

	return Shadow;
}

float DoPSSMShadowMapLookup(const int l, const int slice, float2 tc, float depth)
{
	float Shadow = 0;

#ifndef NOFILTERING
	[branch] if (g_Lights[l].SMFilter == FT_3x3GAUSS)
	{
		for (int y = -1; y <= 1; y++)
		{
			for (int x = -1; x <= 1; x++)
			{
				Shadow += Gauss3x3Weights[x + 1 + (y + 1) * 3] * g_txPSSM[slice].SampleCmpLevelZero(g_sShadowMap, tc + float2(x, y) * g_Lights[l].InvSMSize, depth);					
			}
		}
	}
	else if (g_Lights[l].SMFilter == FT_16xPOISSON)
	{
		for(int i = 0; i < 16; i++)
		{
			Shadow += g_txPSSM[slice].SampleCmpLevelZero(g_sShadowMap, tc + PoissonOffsets[i] * g_Lights[l].InvSMSize, depth);
		}

		Shadow *= 1 / 16.0f;
	}
	else
#endif
	{
		Shadow = g_txPSSM[slice].SampleCmpLevelZero(g_sShadowMap, tc, depth);
	}

	return Shadow;
}

float DoCubeShadowMapLookup(const int l, float3 dir, float depth)
{
	float Shadow = 0;

#ifndef NOFILTERING
	[branch] if (g_Lights[l].SMFilter == FT_3x3GAUSS || g_Lights[l].SMFilter == FT_16xPOISSON)
	{
		//select face
		float3 dirabs = abs(dir);
		float3 xoffsetmask, yoffsetmask;
		if (dirabs.x > dirabs.y && dirabs.x > dirabs.z)
		{
			xoffsetmask = float3(0, 1, 0);
			yoffsetmask = float3(0, 0, 1);
		}
		else if (dirabs.y > dirabs.x && dirabs.y > dirabs.z)
		{
			xoffsetmask = float3(1, 0, 0);
			yoffsetmask = float3(0, 0, 1);
		}
		else
		{
			xoffsetmask = float3(1, 0, 0);
			yoffsetmask = float3(0, 1, 0);
		}

		//assume [-1, +1] goes of whole texture

		[branch] if (g_Lights[l].SMFilter == FT_3x3GAUSS)
		{
			for (int y = -1; y <= 1; y++)
			{
				for (int x = -1; x <= 1; x++)
				{
					float3 offset = xoffsetmask * x * g_Lights[l].InvSMSize * 2 + 
									yoffsetmask * y * g_Lights[l].InvSMSize * 2;
					Shadow += Gauss3x3Weights[x + 1 + (y + 1) * 3] * g_txCubeShadowMaps[l].SampleCmpLevelZero(g_sShadowMap, dir + offset, depth);
				}
			}
		}
		else
		{
			for(int i = 0; i < 16; i++)
			{
				float3 offset = xoffsetmask * PoissonOffsets[i].x * g_Lights[l].InvSMSize * 2+ 
								yoffsetmask * PoissonOffsets[i].y * g_Lights[l].InvSMSize * 2;
				Shadow += g_txCubeShadowMaps[l].SampleCmpLevelZero(g_sShadowMap, dir + offset, depth);
			}

			Shadow *= 1 / 16.0f;
		}

	}
	//else if (g_Lights[l].SMFilter == FT_16xPOISSON)
	//{
	//}
	else
#endif
	{
		Shadow = g_txCubeShadowMaps[l].SampleCmpLevelZero(g_sShadowMap, dir, depth);
	}

	return Shadow;
}

float CalcDirLightShadow(const int l, const float3 WorldPos)
{
	[branch] if (g_Lights[l].SM != 0)
	{
		float Shadow = g_Lights[l].SM;
		if (g_PSSMLightIndex == l)
		{	
			float pointdist = mul(float4(WorldPos, 1), g_matView).z;

			//return pointdist * 0.02;
			//return 1;
			
			//select depth slice
			uint slice = 0;
			for (; slice < g_PSSMSliceCount - 1; slice++)
			{
				if (pointdist > g_PSSMSliceNear[slice].x)
					break;
			}
			//return pointdist * 0.01;
			//return slice * 0.33;

			for (uint i = 0; i < g_PSSMSliceCount & i < MAXSLICES; i++)
			{
				[branch] if (i == slice)
				{
					float4 smtc = mul(float4(WorldPos, 1), g_matPSSM[slice]);
					float3 smtcproj = smtc.xyz / smtc.w;
				
					Shadow *= DoPSSMShadowMapLookup(l, slice, smtc.xy, smtc.z);
				}
			}
		}
		else
		{
			float4 smtc = mul(float4(WorldPos, 1), transpose(g_Lights[l].MatSM));
			float3 smtcproj = smtc.xyz / smtc.w;
		
			Shadow *= Do2DShadowMapLookup(l, smtc.xy, smtc.z);
		} 

		return Shadow;
	}
	else
	{
		return 0;
	}
}

void DoDirLight(const int l, const float3 WorldPos, const float3 Normal, const float3 ReflVec, uniform float Shinyness, inout float3 ioDiffuse, inout float3 ioSpecular)
{
	//vector to light
	float3 LightVec = -g_Lights[l].Direction;

	//shadow
	float Shadow = g_ShadowMapping ? CalcDirLightShadow(l, WorldPos) : 0;

	//calc lighting
	float3 Lighting = g_Lights[l].Color.rgb * (1 - Shadow);

	//do phong lighting
	float2 phong = PhongLighting(Normal, LightVec, ReflVec, Shinyness);

	//apply diffuse and specular lighting
	ioDiffuse += phong.x * Lighting;
	ioSpecular += phong.y * Lighting * g_Lights[l].Specular;
}

float CalcPointLightShadow(const int l, const float3 LightVec, const float LightViewDepth)
{
	[branch] if(g_Lights[l].SM  != 0)
	{
		float4 smdepth = mul(float4(0, 0, LightViewDepth, 1), transpose(g_Lights[l].MatSM));
		
		//float Shadow = g_txCubeShadowMaps[l].SampleCmpLevelZero(g_sShadowMap, -LightVec, LightDist / g_Lights[l].Radius);
		float Shadow = DoCubeShadowMapLookup(l, -LightVec, smdepth.z / smdepth.w);
 
		return Shadow * g_Lights[l].SM;

		//return g_txCubeShadowMaps[l].Sample(g_sShadowMap, -LightVec);
	}
	else
	{
		return 0;
	}
}

void DoPointLight(const int l, const float3 WorldPos, const float3 Normal, const float3 ReflVec, uniform float Shinyness, inout float3 ioDiffuse, inout float3 ioSpecular)
{
	//distance to light
	float3 PosToLight = abs(g_Lights[l].Position - WorldPos);
	float LightViewDepth = max(max(PosToLight.xyz, PosToLight.yxx), PosToLight.zzy);
	float LightDist = length(PosToLight);

	//vector to light
	float3 LightVec = normalize(g_Lights[l].Position - WorldPos);

	//shadow
	float Shadow = g_ShadowMapping ? CalcPointLightShadow(l, LightVec, LightViewDepth) : 0;

	//calculate light attenuation
	float Att = LightAttenuation(l, LightDist);

	//calc lighting
	float3 Lighting = g_Lights[l].Color.rgb * (1 - Shadow) * Att;

	//do phong lighting
	float2 phong = PhongLighting(Normal, LightVec, ReflVec, Shinyness);

	//apply diffuse and specular lighting
	ioDiffuse += phong.x * Lighting;
	ioSpecular += phong.y * Lighting * g_Lights[l].Specular;

	//ioDiffuse = (1 - Shadow);
	//ioDiffuse = LightDist / g_Lights[l].Radius;
	//ioSpecular = 0;
	//ioDiffuse = phong.x * Lighting;
}

float CalcSpotLightShadow(const int l, const float3 WorldPos)
{
	[branch] if (g_Lights[l].SM != 0)
	{
		float4 smtc = mul(float4(WorldPos, 1), transpose(g_Lights[l].MatSM));
		float3 smtcproj = smtc.xyz / smtc.w;

		float Shadow = Do2DShadowMapLookup(l, smtcproj.xy, smtcproj.z);

		return Shadow * g_Lights[l].SM;
	}
	else
	{
		return 0;
	}
}

void DoSpotLight(const int l, const float3 WorldPos, const float3 Normal, const float3 ReflVec, uniform float Shinyness, inout float3 ioDiffuse, inout float3 ioSpecular)
{
	//distance to light
	float LightDist = length(g_Lights[l].Position - WorldPos);	

	//vector to light
	float3 LightVec = normalize(g_Lights[l].Position - WorldPos);

	//shadow
	float Shadow = g_ShadowMapping ? CalcSpotLightShadow(l, WorldPos) : 0;
	
	//attenuation
	float Att = LightAttenuation(l, LightDist);
	
	//angle to spot light direction
	float rho = dot(LightVec, -g_Lights[l].Direction);

	//apply spot light params to attenuation
	//Att *= pow( saturate( (rho - g_Lights[l].SpotParam.y) / (g_Lights[l].SpotParam.x - g_Lights[l].SpotParam.y) ), g_Lights[l].SpotParam.z );
	 if (rho <= g_Lights[l].SpotParam.y) //outside outer radius
		Att = 0.0f;
     else if (rho <= g_Lights[l].SpotParam.x) //outside inner radius
        Att *= pow((rho - g_Lights[l].SpotParam.y) / g_Lights[l].SpotParam.z, g_Lights[l].SpotParam.w);

	//calc lighting
	float3 Lighting = g_Lights[l].Color.rgb * (1 - Shadow) * Att;

	//do phong lighting
	float2 phong = PhongLighting(Normal, LightVec, ReflVec, Shinyness);

	//apply diffuse and specular lighting
	ioDiffuse += phong.x * Lighting;
	ioSpecular += phong.y * Lighting * g_Lights[l].Specular;
}

void DoLighting(const float3 WorldPos, const float3 Normal, const float3 ViewVec, uniform float Shinyness, inout float3 ioDiffuse, inout float3 ioSpecular)
{
	//for (int ilight = 0; ilight < g_LightCount && ilight < MAXLIGHTS; ilight++)
	//{
	//	[branch] if (g_Lights[ilight].Type == LT_DIRECTIONAL)
	//	{
	//		DoDirLight(ilight, WorldPos, Normal, ViewVec, Shinyness, ioDiffuse, ioSpecular);
	//	}
	//	else if (g_Lights[ilight].Type == LT_POINT)
	//	{
	//		DoPointLight(ilight, WorldPos, Normal, ViewVec, Shinyness, ioDiffuse, ioSpecular);
	//	}
	//	else if (g_Lights[ilight].Type == LT_SPOT)
	//	{
	//		DoSpotLight(ilight, WorldPos, Normal, ViewVec, Shinyness, ioDiffuse, ioSpecular);
	//	}
	//	else
	//	{
	//		//parameters are broken
	//		ioDiffuse = float3(1, 1, 0);
	//	}
	//}

	//for (int idir = 0; idir < NUMDIR; idir++)
	//{
	//	DoDirLight(idir, WorldPos, Normal, ViewVec, Shinyness, ioDiffuse, ioSpecular);
	//}

	//for (int ipoint = 0; ipoint < NUMPOINT; ipoint++)
	//{
	//	DoPointLight(NUMDIR + ipoint, WorldPos, Normal, ViewVec, Shinyness, ioDiffuse, ioSpecular);
	//}

	//for (int ispot = 0; ispot < NUMSPOT; ispot++)
	//{
	//	DoSpotLight(NUMDIR + NUMPOINT + ispot, WorldPos, Normal, ViewVec, Shinyness, ioDiffuse, ioSpecular);
	//}

	//calc reflection vector
	float3 ReflVec = normalize(2 * dot(Normal, ViewVec) * Normal - ViewVec);

	//with this loop style we could do shared calucations, eg. calculate the lightdirection for point or spot light
	[unroll] for (int ilight = 0; ilight < NUMLIGHTS; ilight++)
	{
		if (LIGHTTYPE[ilight] == LT_DIRECTIONAL)
		{
			DoDirLight(ilight, WorldPos, Normal, ReflVec, Shinyness, ioDiffuse, ioSpecular);
			//ioDiffuse = float3(1, 0, 0);
			//return;
		}
		else if (LIGHTTYPE[ilight] == LT_POINT)
		{
			DoPointLight(ilight, WorldPos, Normal, ReflVec, Shinyness, ioDiffuse, ioSpecular);
			//ioDiffuse = float3(0, 1, 0);
			//return;
		}
		else if (LIGHTTYPE[ilight] == LT_SPOT)
		{
			DoSpotLight(ilight, WorldPos, Normal, ReflVec, Shinyness, ioDiffuse, ioSpecular);
			//ioDiffuse = float3(0, 0, 1);
			//return;
		}
		else
		{
			//parameters are broken
			ioDiffuse = float3(1, 1, 0);
		}
	}
}

//lighting without specular
void DoLighting(const float3 WorldPos, const float3 Normal, inout float3 ioDiffuse)
{
	//calc reflection vector
	float3 ioSpecular = float3(0, 0, 0);
	float3 ReflVec = float3(0, 0, 0);

	//with this loop style we could do shared calucations, eg. calculate the lightdirection for point or spot light
	[unroll] for (int ilight = 0; ilight < NUMLIGHTS; ilight++)
	{
		if (LIGHTTYPE[ilight] == LT_DIRECTIONAL)
		{
			DoDirLight(ilight, WorldPos, Normal, ReflVec, 0, ioDiffuse, ioSpecular);
			//ioDiffuse = float3(1, 0, 0);
			//return;
		}
		else if (LIGHTTYPE[ilight] == LT_POINT)
		{
			DoPointLight(ilight, WorldPos, Normal, ReflVec, 0, ioDiffuse, ioSpecular);
		}
		else if (LIGHTTYPE[ilight] == LT_SPOT)
		{
			DoSpotLight(ilight, WorldPos, Normal, ReflVec, 0, ioDiffuse, ioSpecular);
		}
		else
		{
			//parameters are broken
			ioDiffuse = float3(1, 1, 0);
		}
	}
}