//define TEXTILE to use texcoords from stream

cbuffer cb
{
	matrix g_matViewProj : VIEWPROJ;
	row_major matrix g_matInvView : INVVIEW;
	row_major matrix g_matInvProj : INVPROJ;
	float3 g_CamPos : CAMPOS;
	float3 g_Ambient : AMBIENT;
	float3 g_Color;
	float  g_Diffuse;
	float  g_Emissive;
	float  g_Stretch;
};

Texture2D g_txParticle;
Texture2D g_txDepth;

SamplerState g_sBilinear
{
    Filter = MIN_MAG_LINEAR_MIP_POINT;
	//Filter = MIN_MAG_MIP_POINT;
    AddressU = Clamp;
    AddressV = Clamp;
};

SamplerState g_sAniso
{
    Filter = ANISOTROPIC;
    AddressU = Clamp;
    AddressV = Clamp;
};

struct VS_IN
{
	float3 Pos : POSITION;
	float  Size : SIZE;
	float  Alpha : ALPHA;
	float  Rotation : ROTATION;

#ifdef TEXTILE
	float4 TexCoords : TEXCOORDS;
#endif
};

struct VS_IN_Oriented
{
	float3 Pos : POSITION;
	float  Size : SIZE;
	float  Alpha : ALPHA;
	float  Rotation : ROTATION;
	float3  Velocity : VELOCITY;

#ifdef TEXTILE
	float4 TexCoords : TEXCOORDS;
#endif
};

struct GS_IN
{
	float3 Pos : POSITION;
	float  Size : TEXCOORD0;
	float  Alpha : TEXCOORD1;
	float  Rotation : TEXCOORD2;
	float4 TexCoords : TEXCOORD3;
};

struct GS_IN_Oriented
{
	float3 Pos : POSITION;
	float  Size : TEXCOORD0;
	float  Alpha : TEXCOORD1;
	float  Rotation : TEXCOORD2;
	float4 TexCoords : TEXCOORD3;
	float3 Orientation : TEXCOORD4;
};

struct PS_IN
{
	float4 Pos : SV_Position;
	float2 TexCoord : TEXCOORD0;
	float2 ScreenCoord : TEXCOORD1;
	float2 Depth : TEXCOORD2;
	float Alpha : TEXCOORD3;
};

GS_IN VS(VS_IN i)
{
	GS_IN o;
    
	o.Pos = i.Pos;
	o.Size = i.Size;
	o.Alpha = i.Alpha;
	o.Rotation = i.Rotation;

#ifdef TEXTILE
	o.TexCoords = i.TexCoords;
#else
	o.TexCoords = float4(0, 0, 1, 1);
#endif
	
    return o;
}

GS_IN_Oriented VSOriented(VS_IN_Oriented i)
{
	GS_IN_Oriented o;
    
	o.Pos = i.Pos;
	o.Size = i.Size;
	o.Alpha = i.Alpha;
	o.Rotation = i.Rotation;

#ifdef TEXTILE
	o.TexCoords = i.TexCoords;
#else
	o.TexCoords = float4(0, 0, 1, 1);
#endif

	o.Orientation = normalize(i.Velocity);
	
    return o;
}

[maxvertexcount(4)]
void GS(point GS_IN i[1], inout TriangleStream<PS_IN> outStream)
{
	PS_IN o;

	float2 rot;
	sincos(i[0].Rotation, rot.x, rot.y);

	float2 scale = i[0].Size;
	scale.y *= (1 + g_Stretch);
	    
   	//build triangle strip
    [unroll] for (int pind = 0; pind < 4; pind++)
    {
		int x = pind & 0x1;
		int y = pind >> 1;

		float2 xcoords = i[0].TexCoords.xz;
		float2 ycoords = i[0].TexCoords.yw;

		o.TexCoord.x = dot(xcoords, float2(x, 1-x));
		o.TexCoord.y = dot(ycoords, float2(y, 1-y));

		float2 vec = (float2(x, 1-y) - 0.5f) * 2 * scale;

		//float2x2 [cos, -sin; sin, cos]
		float2 vecrot = float2(vec.x * rot.y + vec.y * -rot.x,	
								vec.x * rot.x + vec.y * rot.y);
		float3 viewoff = float3(vecrot, 0);
		
		float3 wpos = mul(viewoff, (float3x3)g_matInvView) + i[0].Pos;
		o.Pos = mul(float4(wpos, 1), g_matViewProj);
		
		o.Alpha = i[0].Alpha;

		float2 screencrd = ((o.Pos.xy / o.Pos.w) + 1) * 0.5;
		o.ScreenCoord = float2(screencrd.x, 1 - screencrd.y);

		o.Depth = o.Pos.zw;

        outStream.Append(o);
    }
    outStream.RestartStrip();
}

[maxvertexcount(4)]
void GSOriented(point GS_IN_Oriented i[1], inout TriangleStream<PS_IN> outStream)
{
	PS_IN o;

	float2 rot;
	sincos(i[0].Rotation, rot.x, rot.y);

	float3 viewVec = g_CamPos - i[0].Pos;
	float3 up = normalize(cross(i[0].Orientation, viewVec));

	float2 scale = i[0].Size;
	scale.y *= (1 + g_Stretch);
	    
   	//build triangle strip
    [unroll] for (int pind = 0; pind < 4; pind++)
    {
		int x = pind & 0x1;
		int y = pind >> 1;

		float2 xcoords = i[0].TexCoords.xz;
		float2 ycoords = i[0].TexCoords.yw;

		o.TexCoord.x = dot(xcoords, float2(x, 1-x));
		o.TexCoord.y = dot(ycoords, float2(y, 1-y));
		
		float2 vec = (float2(x, y) - 0.5f) * 2 * scale;
		float2 vecrot = float2(vec.x * rot.y + vec.y * -rot.x,	
								vec.x * rot.x + vec.y * rot.y);
				
		float3 posoff = up * vecrot.x + i[0].Orientation * vecrot.y;
		
		float3 wpos = posoff + i[0].Pos;
		o.Pos = mul(float4(wpos, 1), g_matViewProj);

		float2 screencrd = ((o.Pos.xy / o.Pos.w) + 1) * 0.5;
		o.ScreenCoord = float2(screencrd.x, 1 - screencrd.y);
		o.Depth = o.Pos.zw;
		
		o.Alpha = i[0].Alpha;

        outStream.Append(o);
    }
    outStream.RestartStrip();
}

float4 PS(PS_IN i, uniform bool soft, uniform bool tex) : SV_Target
{
	float4 color = float4(g_Color, 1);

	if (soft)
	{
		float depth = g_txDepth.Sample(g_sBilinear, i.ScreenCoord);
        
  //      if( !bUseMSAA )
		//{
  //          depthSample = g_txDepth.Sample(g_samPoint, screenTex);
  //      } 
		//else
		//{
  //          depthSample = g_txDepthMSAA.Load(int3(int2(screenTex * g_vScreenSize), 0), 0);
  //      }
        
        float4 dscreen = mul(float4(4, 4, depth, 1), g_matInvProj);
        float4 dparticle = mul(float4(0, 0, i.Depth.x / i.Depth.y, 1), g_matInvProj);
        
        float d = dscreen.z / dscreen.w - dparticle.z / dparticle.w;
        if (d < 0)
			discard;
        
		//color.rgb = dscreen.z / dscreen.w * 0.01;
		//color.rgb = i.ScreenCoord.g;
        color.a *= saturate(d / 0.3f); //blend 50cm
	}

	if (tex)
		color *= g_txParticle.Sample(g_sAniso, i.TexCoord);

	//if (dot(color.rgb, float3(1, 1, 1)) == 0)
	//	color.a = 0;
	
	float3 lighting = g_Ambient + g_Diffuse + g_Emissive;
	color.rgb *= lighting;
	color.a *= i.Alpha;
		
	return color;
	//return 1;
}

BlendState AlphaBlending
{
    AlphaToCoverageEnable = FALSE;
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
	//SrcBlend = ONE;
    //DestBlend = INV_SRC_COLOR;
    BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
    RenderTargetWriteMask[0] = 0x0F;
};

//DepthStencilState DisableDepthWrite
//{
//	DepthEnable = true;
//	DepthWriteMask = 0;
//	DepthFunc = LESS;
//	StencilEnable = false;
//};
//
technique10 Render
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(CompileShader(gs_4_0, GS()));
        SetPixelShader(CompileShader(ps_4_0, PS(false, false)));
    }  
}

technique10 Render_Textured
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(CompileShader(gs_4_0, GS()));
        SetPixelShader(CompileShader(ps_4_0, PS(false, true)));
    }  
}

technique10 Render_Oriented
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VSOriented()));
        SetGeometryShader(CompileShader(gs_4_0, GSOriented()));
        SetPixelShader(CompileShader(ps_4_0, PS(false, false)));
    }  
}

technique10 Render_Textured_Oriented
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VSOriented()));
        SetGeometryShader(CompileShader(gs_4_0, GSOriented()));
        SetPixelShader(CompileShader(ps_4_0, PS(false, true)));
    }  
}

technique10 Render_Soft
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(CompileShader(gs_4_0, GS()));
        SetPixelShader(CompileShader(ps_4_0, PS(true, false)));
    }  
}

technique10 Render_Textured_Soft
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(CompileShader(gs_4_0, GS()));
        SetPixelShader(CompileShader(ps_4_0, PS(true, true)));
    }  
}

technique10 Render_Oriented_Soft
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VSOriented()));
        SetGeometryShader(CompileShader(gs_4_0, GSOriented()));
        SetPixelShader(CompileShader(ps_4_0, PS(true, false)));
    }  
}

technique10 Render_Textured_Oriented_Soft
{
    pass p0
    {
        SetBlendState(AlphaBlending, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xFFFFFFFF);
        //SetDepthStencilState(DisableDepthWrite, 0);
        
        SetVertexShader(CompileShader(vs_4_0, VSOriented()));
        SetGeometryShader(CompileShader(gs_4_0, GSOriented()));
        SetPixelShader(CompileShader(ps_4_0, PS(true, true)));
    }  
}