#include "Scattering.fxh"

cbuffer Stuff
{
	float4x4 g_matProj : PROJ;
	float4x4 g_matInvView : INVVIEW;
	float4x4 g_matInvViewProj : INVVIEWPROJ;
	float3 g_CamPos : CAMPOS;
}

struct PS_INPUT
{
    float4 Pos : SV_Position;
    float3 Inscatter : COLOR0;
};

PS_INPUT VS(float2 ScreenPos : SCREENPOS)
{
    PS_INPUT o;

    // transform
    float2 PosOut = ScreenPos * float2(2, -2) - float2(1, -1);
    o.Pos = float4(PosOut, 1, 1);

	float3 v = float3(PosOut / float2(g_matProj[0].x, g_matProj[1].y), 1);
	//float3 v = float3(PosOut, 1);
	v = mul(v, g_matInvView);
	
    float3 ViewDir = normalize(v);
    o.Inscatter = InscatterInf(ViewDir, g_CamPos);
	//o.Inscatter = ViewDir * 0.5 + 0.5;

	return o;
}

// PS
float4 PS(PS_INPUT i) : SV_Target
{
    return float4(i.Inscatter, 1);
}

DepthStencilState DepthEqual
{
	DepthEnable = true;
	DepthFunc = EQUAL;
	DepthWriteMask = 0;
	StencilEnable = false;
};

RasterizerState NoCull
{
    CullMode = None;
    FillMode = Solid;
	//FillMode = WireFrame;

	MultisampleEnable = true;
	AntialiasedLineEnable = true;
};

BlendState NoBlend
{
    AlphaToCoverageEnable = false;
    BlendEnable[0] = false;
	RenderTargetWriteMask[0] = 0x0F;
};

technique10 RenderSky
{
	pass P0
    {      
		SetDepthStencilState(DepthEqual, 0);
		SetRasterizerState(NoCull);
		SetBlendState(NoBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS()));
    }
}

//cbuffer SSAOParam
//{
//	row_major float4x4 g_matProj;
//	row_major float4x4 g_matInvProj;
//	row_major float4x4 g_matView;
//	row_major float4x4 g_matInvView;
//	row_major float4x4 g_matViewProj;
//	row_major float4x4 g_matInvViewProj;
//
//	float2 g_VPSize;
//
//	float g_Radius;
//	float g_Strength;
//
//	int g_SampleCount;
//	int g_NoiseReadMask;
//
//	float4 g_SampleDirs[64]; //dir_xyz
//};
//
//Texture2D g_txDepth;
//Texture2D g_txNormal;
//Texture2D g_txNoise;
//Texture2D g_txSSAO;
//
//SamplerState g_sBilinear
//{
//    Filter = MIN_MAG_LINEAR_MIP_POINT;
//	AddressU = Clamp;
//    AddressV = Clamp;
//};
//
//SamplerState g_sPoint
//{
//	Filter = MIN_MAG_MIP_POINT;
//    AddressU = Clamp;
//    AddressV = Clamp;
//};
//
//struct PS_IN
//{
//	float4 Position : SV_Position;
//	float2 ScreenCoord : TEXCOORD0;
//};
//
//PS_IN VSQuad(in uint vid : SV_VertexID)
//{
//	PS_IN o;
//
//	float tx = vid >> 1; //0, 0, 1, 1
//	float ty = vid & 0x1; //0, 1, 0, 1
//
//	float x = tx * 2.0f - 1.0f;
//	float y = ty * 2.0f - 1.0f;
//		
//	o.Position = float4(x, y, 0, 1);
//	o.ScreenCoord = float2(tx, 1 - ty);
//
//	return o;
//}
//
//float2 Rotation2x2(float2 vec, const float2 sc)
//{
//	//       |cos -sin| 
//	//		 |sin  cos|
//	return float2(vec.x * sc.y + vec.y * -sc.x,	
//				  vec.x * sc.x + vec.y * sc.y);
//}
//
//float4 PSSSAO(PS_IN i) : SV_Target
//{
//	//read non linear depth
//	float depth = g_txDepth.Sample(g_sPoint, i.ScreenCoord);
//
//	if (depth == 1)
//		return 0;
//
//	//modification of implementation at iquilezles.org
//
//	float4 eyepointproj = mul(float4(i.ScreenCoord * float2(2, -2) - float2(1, -1), depth, 1), g_matInvProj);
//	
//
//	//eyepoint
//	float3 eyepoint = eyepointproj.xyz / eyepointproj.w;
//
//	//float4 worldpointproj = mul(eyepointproj, g_matInvView);
//	//float3 worldpoint = worldpointproj.xyz / worldpointproj.w;
//
//	float3 worldnormal = g_txNormal.Sample(g_sPoint, i.ScreenCoord);
//
//	float3 normal = mul(worldnormal, (float3x3)g_matView);
//	float3 tangent = float3(1, 0.5, 0);
//	float3 binormal = normalize(cross(normal, tangent));
//	tangent = cross(binormal, normal);
//
//	//float3 eyenormal = mul(worldnormal, (float3x3)g_matView); 
//	//float3 eyetangent = float3(1, 0, 0);
//	//float3 eyebinormal = normalize(cross(eyenormal, eyetangent));
//	//eyetangent = cross(eyebinormal, eyenormal);
//
//	float frand = g_txNoise.Load(int3(((int)i.Position).x & g_NoiseReadMask, ((int)i.Position.y) & g_NoiseReadMask, 0));
//	float randrot = 3.1415f * 2.0f * frand;
//
//	float2 rot;
//	sincos(randrot, rot.x, rot.y);
//
//	float ao = 0;
//	for (int d = 0; d < g_SampleCount; d++)
//	{
//		float3 dir = g_SampleDirs[d] * g_Radius;
//		float2 sampledirrot = Rotation2x2(dir.xy, rot); //rotate sample points around z axis
//		float3 hemidir = sampledirrot.x * binormal + sampledirrot.y * tangent + dir.z * normal;
//
//		float3 eyesample = eyepoint.xyz + hemidir;
//		float4 sampleproj = mul(float4(eyesample, 1), g_matProj);
//
//		//float4 eyssampleview = mul(float4(eyesample, 1), g_matView);
//		
//		float2 srcd = sampleproj.xy / sampleproj.w * 0.5 + 0.5;
//		srcd.y = 1 - srcd.y;
//		float sampledepth = g_txDepth.SampleLevel(g_sPoint, srcd, 0);
//		float4 sampledepthproj = mul(float4(0, 0, sampledepth, 1), g_matInvProj);
//		sampledepth = sampledepthproj.z /sampledepthproj.w;
//		
//		//return length(hemidir);
//		//return sampledepth * 0.02;
//		//return eyesample.z * 0.02;
//		//return abs(sampledepth - eyesample.z)*10;
//
//		//float occdist = 100 * max(0, eyssampleview.z - sampledepth);
//		//ao += 1 / (1 + occdist * occdist); 
//		float occdist = eyesample.z - sampledepth;
//		if (occdist > 0)// point is occluded
//		{
//			occdist /= g_Radius * 2;
//			//ao += 1;
//			ao += 1 / (1 + occdist * occdist);
//		}
//	}
//
//	return ao / g_SampleCount;
//	//return ao * ao / g_SampleCount / g_SampleCount;
//}
//
//float4 PSBlendSSAO(PS_IN i) : SV_Target
//{
//	float ssao = g_txSSAO.Sample(g_sBilinear, i.ScreenCoord);
//	//return ssao;
//	return float4(0, 0, 0, 1 - (ssao * g_Strength));
//};
//
//BlendState BlendMul
//{
//    AlphaToCoverageEnable = false;
//    BlendEnable[0] = true;
//	RenderTargetWriteMask[0] = 0x0F;
//
//	//multiply dest value with by ao
//	SrcBlend = ZERO;
//    DestBlend = SRC_ALPHA;
//	BlendOp = ADD;
//};
//
//BlendState NoBlend
//{
//    AlphaToCoverageEnable = false;
//    BlendEnable[0] = false;
//	RenderTargetWriteMask[0] = 0x0F;
//};
//
//DepthStencilState NoDepth
//{
//	DepthEnable = false;
//	DepthWriteMask = 0;
//	StencilEnable = false;
//};
//
//RasterizerState NoCull
//{
//    CullMode = None;
//    FillMode = Solid;
//};
//
//technique10 Render_SSAO
//{
//	pass P0
//    {      
//		SetDepthStencilState(NoDepth, 0);
//		SetRasterizerState(NoCull);
//		SetBlendState(NoBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
//    
//        SetVertexShader(CompileShader(vs_4_0, VSQuad()));
//        SetGeometryShader(NULL);
//        SetPixelShader(CompileShader(ps_4_0, PSSSAO()));
//    }
//}
//
//technique10 Render_ApplySSAO
//{
//	pass P0
//    {      
//		SetDepthStencilState(NoDepth, 0);
//		SetRasterizerState(NoCull);
//		SetBlendState(BlendMul, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
//    
//        SetVertexShader(CompileShader(vs_4_0, VSQuad()));
//        SetGeometryShader(NULL);
//        SetPixelShader(CompileShader(ps_4_0, PSBlendSSAO()));
//    }
//}
//
////scattering
//float4x4 g_matInvView;
//float4x4 g_matVP;
//float2 g_ProjMajor;
//float3 g_CamPos;
//float3 g_PointDir;
//float2 g_RTSize;
//
//#include "Shaders\Scattering.fx"
//
////sprite
//float SpriteSize;
//float4 SpriteColor;
//
//texture SpriteTex;
//sampler2D sSprite = sampler_state
//{
//    Texture = <SpriteTex>;
//    AddressU = Clamp;
//    AddressV = Clamp;
//    MipFilter = NONE;
//    MagFilter = LINEAR;
//    MinFilter = LINEAR;
//};
//
//struct VS_OUTPUT
//{
//    float4 Pos : POSITION;
//    float4 Diff : COLOR0;
//};
//
//VS_OUTPUT VS(float2 ScreenPos : POSITION)
//{
//    VS_OUTPUT Out;;
//
//    // transform
//    float2 PosOut = ScreenPos * float2(2, -2) - float2(1, -1) + float2(-1, 1) / g_RTSize;
//    Out.Pos = float4(PosOut, 1, 1);
//
//    float3 v = float3(PosOut / g_ProjMajor, 1);
//    float3 ViewDir = normalize(mul(v, g_matInvView));
//
////	Out.Diff = float4(ViewDir.xyz * 0.5 + 0.5, 1);
//      Out.Diff = float4(InscatterInf(ViewDir, g_CamPos), 1);
//
//	return Out;
//}
//
//// PS
//float4 PS(VS_OUTPUT In) : COLOR
//{
//    return In.Diff;
//}
//
//void VSSprite(out float4 oPos : POSITION, out float oPSize : PSIZE, out float oTex : TEXCOORD0)
//{
//      
//	oPos = float4(mul(g_PointDir*0.999, g_matVP).xyz, 1);
//      //oPos.xy / oPos.w; 
//    oPos.z = sign(oPos.z) * 1;
//      //oPos.w = 1;
//
//	oPSize = SpriteSize;
//
//	oTex = float2(0, 0); //just to avoid warning of d3d
//}
//
//float4 PSSprite(float2 TexC : TEXCOORD0) : COLOR
//{
//	float4 texcol = tex2D(sSprite, TexC);
//	return float4(texcol.rgb * SpriteColor.rgb * SpriteColor.a * 0.6f, texcol.a);
//	return float4(texcol.rgb * 3, texcol.a);
////	return float4(texcol.rgb * SpriteColor.rgb * SpriteColor.a, texcol.a);
//}
//
//float4 PSOne(void) : COLOR
//{
//      return 1;
//}
//
//technique Sky
//{
//    pass P0
//    {
//        AlphaBlendEnable = false;
//        AlphaTestEnable = false;
//
//        ZWriteEnable = false;
//        ZEnable = true;
//	  ZFunc = EQUAL;
//
//        CullMode = None;
//        
//        VertexShader = compile vs_3_0 VS();
//        PixelShader = compile ps_3_0 PS();
//    }
//}
//
//technique SkySprite
//{
//	pass P0
//	{
//		ZEnable = true;
//		ZFunc = EQUAL;
//		ZWriteEnable = false;
//		CullMode = NONE;
//		AlphaBlendEnable = true;
//		AlphaTestEnable = true;
//		AlphaRef = 0;
//		AlphaFunc = GREATER;
//            BlendOp = ADD;
//            SrcBlend = SRCALPHA;
//            DestBlend = INVSRCALPHA;
//		PointSpriteEnable = true;
//
//		VertexShader = compile vs_3_0 VSSprite();
//        PixelShader = compile ps_3_0 PSSprite();
//	}
//
//      //write 1 to alpha channel
//      pass P1
//      {
//            ZEnable = true;
//		ZFunc = EQUAL;
//		ZWriteEnable = false;
//
//		ColorWriteEnable = ALPHA;
//
//		AlphaBlendEnable = false;
//		AlphaTestEnable = false;
//
//            VertexShader = compile vs_3_0 VSSprite();
//            PixelShader = compile ps_3_0 PSOne();
//      }
//}


