#include "Scattering_Shared.fxh"

STScatter Inscatter(float3 Point, float3 CamPos)
{
	//float scale = g_SCBlendToExpFog ? 15000 / g_SCFarPlane : g_SCDistanceScale;
	//float scale = g_SCDistanceScale; 

	float fog = 0;

	//Point *= g_SCDistanceScale;
	//CamPos *= g_SCDistanceScale;
	Point *= g_SCDistanceScale;
	CamPos *= g_SCDistanceScale;	

	//float3 PointDir = (Point - CamPos);
	float3 PointDir = (CamPos - Point);
	float PointDist = length(PointDir);
	PointDir /= PointDist; //normalize point dir

	float PointHeight = max(Point.z, 0);
	float CamHeight = max(CamPos.z, 0);

	//interpolate to camheight, usually 0.5 because scattering is approximated at average, 
	//but at infinite it is taken from camheight
	if (g_SCBlendToFog)
	{
		//float rangeext = saturate(PointDist / (g_SCFarPlane * scale));
		float rangeext = saturate(PointDist * g_SCInvScaledFarPlane);
		fog = pow(rangeext, g_SCFogExp);
		//extinction= lerp(extinction, 0, fog);
		
		PointHeight = lerp(PointHeight, CamHeight, 0.5f + fog * 0.5f);
		PointDist = lerp(PointDist, 15000, fog);
	}
	else
	{
		PointHeight = (PointHeight + CamHeight) * 0.5f;
	}

	//float3 DirPoint = (g_SCSCParam.CamPos - Point) * DimensionScale;
	
	//float PointHeight = Point.z;
	//float PointDist = length(DirPoint);
	//float CosTheta = dot(g_SCSCParam.SunLightDir, DirPoint/PointDist); 
	float CosTheta = dot(g_SCSunLightDir, PointDir); 

	//determine rayleigh and mie optical depth (approximation)
	//scattering strength a height of point
	//float rayPoint = exp(-PointHeightSC);
	//float miePoint = exp(-PointHeightSC * 3);

	//average with scattering at camera height
	//float CamHeight = SC_CamPos.z * DimensionScale;
	//float rayCam = exp(-CamHeightSC);
	//float mieCam = exp(-CamHeightSC * 3);

	//float ray = (rayPoint + rayCam) * 0.5f * PointDist;
	//float mie = (miePoint + mieCam) * 0.5f * PointDist / 3;

	//float ray = exp(-PointHeightSC) * PointDist;
	//float mie = exp(-PointHeightSC * 3) * PointDist / 3;

	float ray = exp(-PointHeight / 15000) * PointDist;
	float mie = exp(-PointHeight / 5000) * PointDist / 3;

	

	//float ray = exp(-max(CamPos.z, 0) / 15000.0f) * 15000.0f;
	//float mie = exp(-max(CamPos.z, 0) / 5000.0f) * 5000.f;

    // calculateextinction terms for inscateredextinction
    float3 extinction = exp(-g_SCBetaRayleigh * ray - g_SCBetaMie * mie); //e^(-beta * depth)
    
	float thetaRay = CosTheta * CosTheta / 2 + 2;

    //calculate mie scatering term
    //Phase2 ( theta ) = (1-g^2) /(1+g^2-2g * cos ( theta ) ) ^(3/2)
    float thetaMie = (1 - g_SCHGsq) / pow(1 + g_SCHGsq - 2 * g_SCHG * CosTheta, 3.f / 2.f );

	// Inscattering ( I ) = ( Beta_R * Phase_R( theta ) + Beta_M * Phase_M( theta ) ) *
    // [1-exp(-Beta_R*s ).exp(-Beta_M*s ) ] / ( Beta_R + Beta_M)
	float3 inscatter = (g_SCBetaDashRayleigh * thetaRay + g_SCBetaDashMie * thetaMie) * (1 - extinction) / (g_SCBetaRayleigh + g_SCBetaMie);
    inscatter = saturate(inscatter);
    inscatter *= g_SCSunColor;
	inscatter += INSCATTERBIAS;

	if (g_SCBlendToFog)
	{
	//	float rangeext = saturate(PointDist / (g_SCFarPlane * scale));
	//	float fog = pow(rangeext, 8);
		extinction= lerp(extinction, 0, fog);
	}

    STScatter o;

	o.Inscatter = inscatter;
	o.Extinction = extinction;
	//o.Extinction = 0;
	
	//o.Extinction = CosTheta;
	//o.Extinction = PointDist / g_SCDistanceScale / g_SCFarPlane;
	//o.Inscatter = g_SCSunColor;

	return o;
}

float3 InscatterInf(const float3 ViewDir, float3 CamPos)
{
	//float scale = g_SCBlendToExpFog ? 15000 / g_SCFarPlane : g_SCDistanceScale;
	//float scale = g_SCDistanceScale; 

	CamPos *= g_SCDistanceScale;

	float CosTheta = dot(g_SCSunLightDir, -ViewDir);
    //float thetaRay = cosTheta * cosTheta + 1;
    float thetaRay = CosTheta * CosTheta / 2 + 2;

	//return saturate(cosTheta);
	//return saturate(thetaRay);

	float CamHeightSC = max(CamPos.z, 0);

	//real:
    //float distRayleigh = 15000 * exp(-max(CamPos.z, 0) / 15000.0f); //*PointDist (=15000)
    //float distMie = 5000 * exp(-max(CamPos.z, 0) / 5000.0f); //*PointDist / 3 (=5000)

	//to be same with objects and terrain:
	//float ray = exp(-max(CamPos.z, 0) / 15000.0f) * 15000;
	//float mie = exp(-max(CamPos.z, 0) / 5000.0f) * 5000;
	float ray = exp(-CamHeightSC / 15000) * 15000;
	float mie = exp(-CamHeightSC / 5000) * 5000;

	//distRayleigh *= DimensionScale;
	//distMie *= DimensionScale;

    // calculateextinction terms for inscateredextinction
    float3 extinction = exp(-g_SCBetaRayleigh * ray - g_SCBetaMie * mie);

	//return float4(extinction, 1);
    
    //calculate mie scatering term
    //Phase2 ( theta ) = (1-g^2) /(1+g^2-2g * cos ( theta ) ) ^(3/2)
    float thetaMie = (1 - g_SCHGsq) / pow(1 + g_SCHGsq - 2 * g_SCHG * CosTheta, 3.f / 2.f );

	//return saturate(thetaMie);

    // Inscattering ( I ) = ( Beta_R * Phase_R( theta ) + Beta_M * Phase_M( theta ) ) *
    // [1-exp(-Beta_R*s ).exp(-Beta_M*s ) ] / ( Beta_R + Beta_M)
    float3 inscatter = (g_SCBetaDashRayleigh * thetaRay + g_SCBetaDashMie * thetaMie) * (1 - extinction) / (g_SCBetaRayleigh + g_SCBetaMie);	
    inscatter = saturate(inscatter);
    inscatter *= g_SCSunColor;
	inscatter += INSCATTERBIAS;

	//inscatter = 0;
	//return g_SCSCParam.SunColorIntensity.w;

	//STInscatter o;
	//o.Inscatter = inscatter + InscatterBias;
	//o.Extinction = 0;
	//return o;

	return inscatter;
}

//
//float4 PS(PS_IN i) : SV_Target
//{
//	//read non linear depth
//	float depth = g_SCtxDepth.Sample(g_SCsBilinear, i.ScreenCoord);
//
//	STInscatter scatter;
//
//	[branch] if (depth == 1)
//	{
//		float3 dir = normalize(mul(float3(i.ScreenCoord * float2(2, -2) - float2(1, -1), 1), g_SCmatInvViewProj));
//		scatter = InscatterInf(dir, g_SCCamPos);
//	}
//	else
//	{
//		float4 pos = mul(float4(i.ScreenCoord * float2(2, -2) - float2(1, -1), depth, 1), g_SCmatInvViewProj);
//		float3 wpos = pos.xyz / pos.w;
//
//		scatter = Inscatter(wpos, g_SCCamPos);
//	}
//
//	return float4(scatter.Inscatter, 1);
//};
//
