//include shader for skinned geometry
//maximum 4 skinweights are supported per vertex
//define NUMWEIGHTS to set number of used skinweights

#define VERTEXSKININFO					\
	float4 SkinWeights : SKINWEIGHTS;	\
	uint4 BoneIndices : BONEINDICES;

#ifndef NUMWEIGHTS
#define NUMWEIGHTS 4
#endif

#include "Skinning_Shared.fxh"

matrix GetBoneTransform(uint iBone)
{
	iBone *= 4;
    float4 row1 = g_txBoneMatrices.Load(int2(iBone + 0, 0));
    float4 row2 = g_txBoneMatrices.Load(int2(iBone + 1, 0));
    float4 row3 = g_txBoneMatrices.Load(int2(iBone + 2, 0));
    float4 row4 = g_txBoneMatrices.Load(int2(iBone + 3, 0));
    
    return matrix(row1, row2, row3, row4);
}

void SkinVert(float4 SkinWeights, uint4 BoneIndices, inout float4 Pos)
{
	uint numweights = NUMWEIGHTS;
	
	if (numweights > 0)
	{
		matrix m = GetBoneTransform(BoneIndices.x);
		float4 p = SkinWeights.x * mul(Pos, m);
		
		if (numweights > 1)
		{
			m = GetBoneTransform(BoneIndices.y);
			p += SkinWeights.y * mul(Pos, m);
			
			if (numweights > 2)
			{
				m = GetBoneTransform(BoneIndices.z);
				p += SkinWeights.z * mul(Pos, m);
				
				if (numweights > 3)
				{
					m = GetBoneTransform(BoneIndices.w);
					p += SkinWeights.w * mul(Pos, m);
				}
			}
		}
		
		Pos = p;
	}
}

void SkinVert(float4 SkinWeights, uint4 BoneIndices, inout float4 Pos, inout float3 Normal)
{
	uint numweights = NUMWEIGHTS;
		
	if (numweights > 0)
	{
		matrix m = GetBoneTransform(BoneIndices.x);
		float4 p = SkinWeights.x * mul(Pos, m);
		float3 n = SkinWeights.x * mul(Normal, (float3x3)m);
		
		if (numweights > 1)
		{
			m = GetBoneTransform(BoneIndices.y);
			p += SkinWeights.y * mul(Pos, m);
			n += SkinWeights.y * mul(Normal, (float3x3)m);
			
			if (numweights > 2)
			{
				m = GetBoneTransform(BoneIndices.z);
				p += SkinWeights.z * mul(Pos, m);
				n += SkinWeights.z * mul(Normal, (float3x3)m);
				
				if (numweights > 3)
				{
					m = GetBoneTransform(BoneIndices.w);
					p += SkinWeights.w * mul(Pos, m);
					n += SkinWeights.w * mul(Normal, (float3x3)m);
				}
			}
		}
		
		Pos = p;
		Normal = n;
	}
}

void SkinVert(float4 SkinWeights, uint4 BoneIndices, inout float4 Pos, inout float3 Normal, inout float3 Tangent)
{
	uint numweights = NUMWEIGHTS;
	
	if (numweights > 0)
	{
		row_major matrix m = GetBoneTransform(BoneIndices.x);
		float4 p = SkinWeights.x * mul(Pos, m);
		float3 n = SkinWeights.x * mul(Normal, (float3x3)m);
		float3 t = SkinWeights.x * mul(Tangent, (float3x3)m);
		
		if (numweights > 1)
		{
			m = GetBoneTransform(BoneIndices.y);
			p += SkinWeights.y * mul(Pos, m);
			n += SkinWeights.y * mul(Normal, (float3x3)m);
			t += SkinWeights.y * mul(Tangent, (float3x3)m);
			
			if (numweights > 2)
			{
				m = GetBoneTransform(BoneIndices.z);
				p += SkinWeights.z * mul(Pos, m);
				n += SkinWeights.z * mul(Normal, (float3x3)m);
				t += SkinWeights.z * mul(Tangent, (float3x3)m);
				
				if (numweights > 3)
				{
					m = GetBoneTransform(BoneIndices.w);
					p += SkinWeights.w * mul(Pos, m);
					n += SkinWeights.w * mul(Normal, (float3x3)m);
					t += SkinWeights.w * mul(Tangent, (float3x3)m);
				}
			}
		}
		
		Pos = p;
		Normal = n;
		Tangent = t;
	}
}
