cbuffer cb
{
	row_major matrix g_matProj : PROJ;
	row_major matrix g_matView : VIEW;
	float3			 g_Ambient : AMBIENT;
	float3			 g_Diffuse;
};

TextureCube g_txSkyCube;

shared SamplerState g_sAniso
{
    Filter = ANISOTROPIC;
    MaxAnisotropy = 16;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

struct VS_IN
{
    float3 Pos : POSITION;
};

struct PS_IN
{
    float4 Pos : SV_Position;
	float3 Normal : TEXCOORD0;
};

PS_IN VSSky(VS_IN i)
{
    PS_IN o;

	o.Normal = i.Pos;
	float3 v = mul(i.Pos, (float3x3)g_matView);
	o.Pos = mul(float4(v, 1), g_matProj);
	o.Pos.z = o.Pos.w;
	o.Pos.z -= 0.00001f;
	
    return o;
}

float4 PSSky(PS_IN i) : SV_Target
{
	float3 col = g_txSkyCube.Sample(g_sAniso, i.Normal).rgb * (g_Diffuse + g_Ambient);

	return float4(col, 1);
}

RasterizerState RasNoCull
{
    CullMode = Front;
    FillMode = Solid;

	MultisampleEnable = true;
	AntialiasedLineEnable = true;
};

DepthStencilState DSState
{
	DepthEnable = true;
	DepthWriteMask = 0;
	DepthFunc = LESS_EQUAL;
	StencilEnable = false;
};

BlendState NoBlend
{
    BlendEnable[0] = FALSE;
};

technique10 RenderSky
{
	pass P0
    {      
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(NoBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSSky()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSSky()));
    }
}
