#include "Lighting.fxh"

#include "Terrain_Shared.fxh"

#include "Scattering.fxh"

float GetLOD(const float3 campos, const float3 pos)
{
	float d = length(campos - pos) / (g_PatchSize * g_UnitSize);

	float lod = log2(max(0.00001f, d - g_PatchOffsetScale.w / g_UnitSize * 0.7f));
	return g_Tiling - min(g_Tiling, max(0, lod));
	//return g_Tiling - lod;
}

PS_IN VSBlend(int2 GridPos : POSITION)
{
	PS_IN o;

	float height = g_txHeightmap.Load(int3(GridPos + int2(1, 1), 0)).r; //move + (1,1) because texture contains one addtional border row
	float2 nxy = g_txNormal.Load(int3(GridPos, 0)).rg;
	
	float3 pos0 = float3(GridPos * g_PatchOffsetScale.w, height) + g_PatchOffsetScale.xyz;

	float lod = GetLOD(g_CamPos, pos0); 
	float s = 1 - min(max(0, (g_LODSizePatchPos.x - lod) * 2 - 1), 1);

	[branch] if (s > 0)
	{
		//coordinates to sample next level
		float2 gridposnext = float2(GridPos) * 0.5f + g_LODSizePatchPos.zw;
		float2 nextheighttc = (gridposnext + 1.5f) / (g_PatchSize + 3.0f);
		float2 nextnormaltc = (gridposnext + 0.5f) / (g_PatchSize + 1.0f);

		float heightnext = g_txHeightmapNext.SampleLevel(g_sBilinear, nextheighttc, 0).r;
		float2 nxynext = g_txNormalNext.SampleLevel(g_sBilinear, nextnormaltc, 0).rg;
    
		height = height + (heightnext - height) * s;
		nxy = nxy + (nxynext - nxy) * s;
	}
	//height = lod;
	//height = length(g_CamPos - pos0) * 0.01;

	float3 normal = float3(nxy, sqrt(1 - dot(nxy, nxy))); //1 - x^2 - y^2
	float3 pos = float3(GridPos * g_PatchOffsetScale.w, height) + g_PatchOffsetScale.xyz;

	//normal.z = s < 0.001 ? 0 : (s > 0.999 ? 1 : 0.5);
	//normal.z = s;
	//normal.z = lod * 0.33;
	//normal.z = g_LODSizePatchPos.x * 0.33;
	//normal.z = g_Tiling * 0.03f;

	o.PosT = mul(float4(pos, 1), g_matVP);
	o.Normal = normal;
	o.Pos = pos;
	o.MaskCoord = GridPos / (float)g_PatchSize;
	o.LodBlend = s;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
		
	return o;
}

PS_IN VS(int2 GridPos : POSITION)
{
	PS_IN o;

	float height = g_txHeightmap.Load(int3(GridPos + int2(1, 1), 0)).r; //move + (1,1) because texture contains one addtional border row
	float2 nxy = g_txNormal.Load(int3(GridPos, 0)).rgb;
	float3 normal = float3(nxy, sqrt(1 - dot(nxy, nxy))); //1 - x^2 - y^2
	//float3 normal = float3(0, 0, 1);

	float3 pos = float3(GridPos * g_PatchOffsetScale.w, height) + g_PatchOffsetScale.xyz;

	o.PosT = mul(float4(pos, 1), g_matVP);
	o.Normal = normal;
	o.Pos = pos;
	o.MaskCoord = GridPos / (float)g_PatchSize;
	o.LodBlend = 0;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
		
	return o;
}

float4 PS(PS_IN i) : SV_Target
{   
	//read blend texture from bottom to top and do blending of diffuse textures

	float3 diffuse = g_txLayers[0].Sample(g_sAniso, i.Pos.xy * g_TexScales[0]);

	[unroll] for (int l = 0; l < MAXLAYERS && l < g_LayerCount; l++)
	{ 
		//uint l = 0;
		float ms = (g_MaskRes[l] - 3) / g_MaskRes[l];
		float blend = g_txMasks[l].Sample(g_sBilinear, i.MaskCoord * ms + 1.5f / g_MaskRes[l]).r;
		//todo try pre calculating gradients and branch texture read
		//[branch] if (blend > 0)
		{
			float3 color = g_txLayers[l + 1].Sample(g_sAniso, i.Pos.xy * g_TexScales[l + 1]).rgb;
			diffuse = diffuse * (1 - blend) + blend * color;
		}
		//return g_TexScales[l+1] * 100;
	}

	//return float4(i.MaskCoord.xy, 0, 0);

	float3 ViewVec = normalize(g_CamPos - i.Pos);
	//float3 normal = i.Normal;
	float3 normal = normalize(i.Normal);

	float3 difflight = g_AmbientLight;

	DoLighting(i.Pos, normal, difflight);

	diffuse *= difflight;

	if (g_SCEnabled)
		diffuse.rgb = diffuse.rgb * i.Extinction + i.Inscatter;
	//else
	//	diffuse.rgb = 0.5f;

	//diffuse.rgb = i.Extinction;
	//diffuse.rgb = i.Inscatter;

	//return float4(difflight, 1);
	//return i.Normal.z;
	return float4(diffuse, 1);
}

technique10 Render
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS()));
    }  
}

technique10 RenderLODBlend
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSBlend()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS()));
    }  
}