cbuffer cbPatch
{
	float4  g_PatchOffsetScale; //offset = xyz, scale = w
	//lod, psize, position of patch in next lod
	//int4	g_LODSizePatchPos;
};

Texture2D g_txHeightmap;
Texture2D g_txHeightmapNext;
Texture2D g_txNormal;
Texture2D g_txNormalNext;

//define vertex input
struct TerrainVertex
{
	int2 GridPos : POSITION;
};
#define SM_INPUT TerrainVertex

//define function to get the position
float3 GetPos(TerrainVertex i)
{
#ifdef BLEND
	float height = g_txHeightmap.Load(int3(GridPos + int2(1, 1), 0)).r; //move + (1,1) because texture contains one addtional border row
	float3 pos0 = float3(GridPos * g_PatchOffsetScale.w, height) + g_PatchOffsetScale.xyz;

	float lod = GetLOD(g_CamPos, pos0); 
	float s = 1 - min(max(0, (g_LODSizePatchPos.x - lod) * 2 - 1), 1);

	[branch] if (s > 0)
	{
		//coordinates to sample next level
		float2 gridposnext = float2(GridPos) * 0.5f + g_LODSizePatchPos.zw;
		float2 nextheighttc = (gridposnext + 1.5f) / (g_PatchSize + 3.0f);
		float heightnext = g_txHeightmapNext.SampleLevel(g_sBilinear, nextheighttc, 0).r;
		height = height + (heightnext - height) * s;
	}
	
	float3 pos = float3(GridPos * g_PatchOffsetScale.w, height) + g_PatchOffsetScale.xyz;
#else
	float height = g_txHeightmap.Load(int3(i.GridPos + int2(1, 1), 0)).r; //move + (1,1) because texture contains one addtional border row
	
	float3 pos = float3(i.GridPos * g_PatchOffsetScale.w, height) + g_PatchOffsetScale.xyz;
#endif

	return pos;
}
#define SM_GETPOSITION GetPos

float3 GetNormal(TerrainVertex i)
{
#ifdef BLEND
	float height = g_txHeightmap.Load(int3(GridPos + int2(1, 1), 0)).r; //move + (1,1) because texture contains one addtional border row
	float2 nxy = g_txNormal.Load(int3(GridPos, 0)).rg;
	
	float3 pos0 = float3(GridPos * g_PatchOffsetScale.w, height) + g_PatchOffsetScale.xyz;

	float lod = GetLOD(g_CamPos, pos0); 
	float s = 1 - min(max(0, (g_LODSizePatchPos.x - lod) * 2 - 1), 1);

	[branch] if (s > 0)
	{
		//coordinates to sample next level
		float2 gridposnext = float2(GridPos) * 0.5f + g_LODSizePatchPos.zw;
		float2 nextnormaltc = (gridposnext + 0.5f) / (g_PatchSize + 1.0f);
		float2 nxynext = g_txNormalNext.SampleLevel(g_sBilinear, nextnormaltc, 0).rg;
		nxy = nxy + (nxynext - nxy) * s;
	}
#else
	float2 nxy = g_txNormal.Load(int3(i.GridPos, 0)).rg;
#endif

	return float3(nxy, sqrt(1 - dot(nxy, nxy))); //1 - x^2 - y^2
}
#define SM_GETNORMAL GetNormal

#include "ShadowMapping.fx"