//shader that is used to render util stuff
//the input geometry must already be transformed to world space

struct PS_COLOR_IN
{
	float4 PosT		: SV_Position;
	float4 Color	: TEXCOORD0;
};

struct PS_SHADED_IN
{
	float4 PosT		: SV_Position;
	float4 Color	: TEXCOORD0;
	float3 Pos		: TEXCOORD1;
	float3 Normal	: NORMAL;
};

cbuffer cb
{
    matrix g_matWVP : WORLDVIEWPROJ;
    matrix g_matWorld : WORLD;
    float4 g_Color;    
    float3 g_CamPos;
    float g_SpecularExp = 32;
    float3 g_LightColor;
	float3 g_LightDir;
    float3 g_AmbientLight;
    float2 g_VPSize : VPSIZE;
};

PS_COLOR_IN VSPosT(float3 Pos : POSITION)
{
	PS_COLOR_IN o;
	float2 xy = (Pos.xy / g_VPSize * 2 - 1) * float2(1, -1);
	o.PosT = float4(xy, Pos.z, 1);
	o.Color = g_Color;
	return o;
}

PS_COLOR_IN VSPos(float3 Pos : POSITION)
{
	PS_COLOR_IN o;
	o.PosT = mul(float4(Pos, 1), transpose(g_matWVP));
	o.Color = g_Color;
	return o;
}

PS_COLOR_IN VSPosVColor(float3 Pos : POSITION, float4 vColor : COLOR)
{
	PS_COLOR_IN o;
	o.PosT = mul(float4(Pos, 1), transpose(g_matWVP));
	o.Color = vColor * g_Color;
	return o;
}

PS_SHADED_IN VSPosNormal(float3 Pos : POSITION, float3 Normal : NORMAL)
{
	PS_SHADED_IN o;
	
	o.Pos = mul(float4(Pos, 1), transpose(g_matWorld));
    o.PosT = mul(float4(Pos, 1), transpose(g_matWVP));
    o.Normal = mul(Normal, transpose((float3x3)g_matWorld));
    o.Color = g_Color;
    
    return o;
}

float4 PSColor(PS_COLOR_IN i) : SV_Target
{   
	return i.Color;
}

float4 PSShaded(PS_SHADED_IN i) : SV_Target
{
	float4 result = g_Color;
	
	float3 n = normalize(i.Normal);
	
	float3 LightVec = -g_LightDir;
	
	float l = saturate(dot(n, LightVec));
	
	result.rgb  *= g_AmbientLight + l * g_LightColor;
	
	if (g_SpecularExp > 0)
    {
		float3 ViewVec = normalize(g_CamPos - i.Pos);
    
		float3 HalfVec = normalize(ViewVec + LightVec);
		float3 specular = pow(saturate(dot(HalfVec, n)), g_SpecularExp) * g_LightColor;
		//float3 ReflVec = normalize(2 * l * n - LightVec);
		//float3 specular = pow(saturate(dot(ReflVec, ViewVec)), g_SpecularExp) * g_LightColor;
		
		result.rgb += specular; // * g_Specular;
	}
	    
    return result * i.Color;
}

RasterizerState RasState
{
    //CullMode = Back;
	CullMode = None;
    FillMode = Solid;
	MultisampleEnable = true;
	AntialiasedLineEnable = true;
};

DepthStencilState DSState
{
	DepthEnable = true;
	DepthWriteMask = 0;
	DepthFunc = LESS;
	StencilEnable = false;
};

BlendState NoBlend
{
    BlendEnable[0] = FALSE;
};

BlendState AlphaBlend
{
    BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
    BlendOp = ADD;
};

technique10 Render_PosT_Colored
{
    pass p0
    {
		//SetDepthStencilState(DSState, 0);
		//SetRasterizerState(RasState);
		SetBlendState(NoBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSPosT()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSColor()));
    }  
}

technique10 Render_Colored
{
    pass p0
    {
		//SetDepthStencilState(DSState, 0);
		//SetRasterizerState(RasState);
		SetBlendState(NoBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSPos()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSColor()));
    }  
}

technique10 Render_Colored_Alpha
{
    pass p0
    {
		//SetDepthStencilState(DSState, 0);
		//SetRasterizerState(RasState);
		SetBlendState(AlphaBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSPos()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSColor()));
    }  
}

technique10 Render_VColored
{
    pass p0
    {
		//SetDepthStencilState(DSState, 0);
		//SetRasterizerState(RasState);
		SetBlendState(NoBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSPosVColor()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSColor()));
    }  
}

technique10 Render_VColored_Alpha
{
    pass p0
    {
		//SetDepthStencilState(DSState, 0);
		//SetRasterizerState(RasState);
		SetBlendState(AlphaBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSPosVColor()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSColor()));
    }  
}

technique10 Render_Shaded
{
    pass p0
    {
		//SetDepthStencilState(DSState, 0);
		//SetRasterizerState(RasState);
		SetBlendState(AlphaBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSPosNormal()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSShaded()));
    }  
}

technique10 Render_Shaded_Alpha
{
    pass p0
    {
		//SetDepthStencilState(DSState, 0);
		//SetRasterizerState(RasState);
		SetBlendState(AlphaBlend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSPosNormal()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PSShaded()));
    }  
}
