#include "XMaterial_Shared.fxh"

#include "Lighting.fxh"

#include "Skinning.fxh"

#include "Scattering.fxh"

//use this define to render material double sided
//#define DOUBLESIDED 0
//#define ALPHABLEND 0
//#define ALPHATEST 0

PS_IN VS(float3 Pos : POSITION, float3 Normal : NORMAL)
{
	PS_IN o;
    
    o.Pos = mul(float4(Pos, 1), g_matWorld);
    o.PosT = mul(float4(Pos, 1), g_matWVP);
    o.Normal = mul(Normal, (float3x3)g_matNormal); // maybe have to be renormalize, because the input mesh might not have normals with the same length
    o.TexCoord = float2(0, 0);
	o.Color = 0;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
	
	return o;
}

PS_IN VSSkinned(float3 Pos : POSITION, float3 Normal : NORMAL, float4 SkinWeights : SKINWEIGHTS, uint4 BoneIndices : BONEINDICES)
{
	PS_IN o;
    
    float4 p = float4(Pos, 1);
    float3 n = Normal;
    
    SkinVert(SkinWeights, BoneIndices, p, n);
	
	o.Pos = p;
    o.PosT = mul(p, g_matVP);
    o.Normal = n;
    o.TexCoord = float2(0, 0);
	o.Color = 0;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
	
    return o;
}

PS_IN VSTex(float3 Pos : POSITION, float3 Normal : NORMAL, float2 TexCoord : TEXCOORD)
{
	PS_IN o;
    
    o.Pos = mul(float4(Pos, 1), g_matWorld);
    o.PosT = mul(float4(Pos, 1), g_matWVP);
    o.Normal = mul(Normal, (float3x3)g_matNormal);
    o.TexCoord = TexCoord;
    o.Color = 0;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
    
    return o;
}

PS_IN VSTexSkinned(float3 Pos : POSITION, float3 Normal : NORMAL, float2 TexCoord : TEXCOORD, float4 SkinWeights : SKINWEIGHTS, uint4 BoneIndices : BONEINDICES)
{
	PS_IN o;
    
    float4 p = float4(Pos, 1);
    float3 n = Normal;
    
    SkinVert(SkinWeights, BoneIndices, p, n);
    
    o.Pos = p;
    o.PosT = mul(p, g_matVP);
    o.Normal = n;
    o.TexCoord = TexCoord;
    o.Color = 0;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
    
    return o;
}

PS_IN VSCol(float3 Pos : POSITION, float3 Normal : NORMAL, float4 Color : COLOR)
{
	PS_IN o;
    
    o.Pos = mul(float4(Pos, 1), g_matWorld);
    o.PosT = mul(float4(Pos, 1), g_matWVP);
    o.Normal = mul(Normal, (float3x3)g_matNormal);
    o.TexCoord = float2(0, 0);
    o.Color = Color;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
    
    return o;
}

PS_IN VSColSkinned(float3 Pos : POSITION, float3 Normal : NORMAL, float4 Color : COLOR, float4 SkinWeights : SKINWEIGHTS, uint4 BoneIndices : BONEINDICES)
{
	PS_IN o;
    
    float4 p = float4(Pos, 1);
    float3 n = Normal;
    
    SkinVert(SkinWeights, BoneIndices, p, n);
    
    o.Pos = p;
    o.PosT = mul(p, g_matVP);
    o.Normal = n;
    o.TexCoord = float2(0, 0);
    o.Color = Color;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
    
    return o;
}

PS_IN VSTexCol(float3 Pos : POSITION, float3 Normal : NORMAL, float2 TexCoord : TEXCOORD, float4 Color	: COLOR)
{
	PS_IN o;
    
    o.Pos = mul(float4(Pos, 1), g_matWorld);
    o.PosT = mul(float4(Pos, 1), g_matWVP);
    o.Normal = mul(Normal, (float3x3)g_matNormal);
    o.TexCoord = TexCoord;
    o.Color = Color;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
    
    return o;
}

PS_IN VSTexColSkinned(float3 Pos : POSITION, float3 Normal : NORMAL, float2 TexCoord : TEXCOORD, float4 Color : COLOR, float4 SkinWeights : SKINWEIGHTS, uint4 BoneIndices : BONEINDICES)
{
	PS_IN o;
	
	float4 p = float4(Pos, 1);
    float3 n = Normal;
    
    SkinVert(SkinWeights, BoneIndices, p, n);
    
    o.Pos = p;
    o.PosT = mul(p, g_matVP);
    o.Normal = n; 
    o.TexCoord = TexCoord;
    o.Color = Color;

	STScatter scatter;
	[branch] if (g_SCEnabled)
		scatter = Inscatter(o.Pos, g_CamPos);
	o.Inscatter = scatter.Inscatter;
	o.Extinction = scatter.Extinction;
    
    return o;
}

float4 PS(PS_IN i, uint vFace : SV_IsFrontFace, uniform bool bTex, uniform bool bVCol) : SV_Target
{   
	//return i.Color;
	
	//return float4(g_AmbientLight, 1);

	float4 color = 1;
    
    if (bTex)
		color *= g_txDiffuse.Sample(g_sTexture, i.TexCoord);
		
	if (bVCol)
		color *= i.Color;

	color.a *= g_Diffuse.a;

#ifdef ALPHATEST
	if (color.a < g_AlphaRef)
		discard;
#endif

	float3 normal = normalize(i.Normal);

#ifdef DOUBLESIDED
	if (vFace <= 0)
		normal = -normal;
#endif

	float3 ViewVec = normalize(g_CamPos - i.Pos);

	//return float4(normalize(g_CamPos), 0.5f);
	//return float4(ViewVec, 1);

	const float WRAP = 0.5f;
	float3 difflight = (1 + saturate(normal.z + 1 - WRAP) / (2 - WRAP)) * g_AmbientLight * 0.5f;
	float3 speclight = 0;

	DoLighting(i.Pos, normal, ViewVec, g_Shinyness, difflight, speclight);

	//return g_PSSMLightIndex * 0.5;
	//return float4(difflight, 1);
	//float pointdist = mul(float4(i.Pos, 1), g_matView).z;
	//return pointdist * 0.02;

	//return 0;
	//float pointdist = mul(float4(i.Pos, 1), g_matView).z;
	//return float4(pointdist * 0.01, 0, 0, 1);

	difflight *= g_Diffuse.rgb;
	speclight *= g_Specular.rgb;

	color.rgb *= difflight;
	color.rgb += speclight + g_Emissive;

	if (g_SCEnabled)
		color.rgb = color.rgb * i.Extinction + i.Inscatter;

	//return float4(difflight, 1);
	//return diffuse.a;
	return color;
	//return float4(i.Pos.xy / 80, 0, 1);
}

RasterizerState RasNoCull
{
#ifdef DOUBLESIDED
    CullMode = None;
#else
	CullMode = Back;
#endif

    FillMode = Solid;
	MultisampleEnable = true;
	AntialiasedLineEnable = true;
};

DepthStencilState DSState
{
	DepthEnable = true;
#ifdef ALPHABLEND
	DepthWriteMask = 0;
#else
	DepthWriteMask = ALL;
#endif
	DepthFunc = LESS;
	StencilEnable = false;
};

BlendState Blend
{
	AlphaToCoverageEnable = FALSE;
	RenderTargetWriteMask[0] = 0x0F;

#ifdef ALPHABLEND
	BlendEnable[0] = TRUE;
    SrcBlend = SRC_ALPHA;
    DestBlend = INV_SRC_ALPHA;
	BlendOp = ADD;
    SrcBlendAlpha = ZERO;
    DestBlendAlpha = ZERO;
    BlendOpAlpha = ADD;
#else
    BlendEnable[0] = FALSE;
#endif
};

technique10 Render
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VS()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(false, false)));
    }  
}

technique10 Render_Textured
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSTex()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(true, false)));
    }  
}

technique10 Render_Colored
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSCol()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(false, true)));
    }  
}

technique10 Render_Textured_Colored
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSTexCol()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(true, true)));
    }  
}

technique10 Render_Skinned
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSSkinned()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(false, false)));
    }  
}

technique10 Render_Textured_Skinned
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSTexSkinned()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(true, false)));
    }  
}

technique10 Render_Colored_Skinned
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSColSkinned()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(false, true)));
    }  
}

technique10 Render_Textured_Colored_Skinned
{
    pass p0
    {
		SetDepthStencilState(DSState, 0);
		SetRasterizerState(RasNoCull);
		SetBlendState(Blend, float4(0.0f, 0.0f, 0.0f, 0.0f), 0xffffffff);
    
        SetVertexShader(CompileShader(vs_4_0, VSTexColSkinned()));
        SetGeometryShader(NULL);
        SetPixelShader(CompileShader(ps_4_0, PS(true, true)));
    }  
}